@@ -9,7 +9,6 @@ from __future__ import (division, print_function, absolute_import,
 import numpy as np
 import scipy.optimize as so
 from scipy.integrate import simps, trapz
-from p_winds import tools
 from astropy import units as u, constants as c
 
 __all__ = ["average_molecular_weight", "sound_speed", "radius_sonic_point",
@@ -208,12 +207,10 @@ def structure(r, v_guess=None):
     # guess is 2.0. This is a hacky solution, but it seems to work well.
     elif isinstance(r, np.ndarray):
         # If r is a ``numpy.ndarray``, we do a dirty little hack to setup an
-        # array of first guesses `x0` whose values are 0.1 for r <= 1, and 2 if
-        # r > 1.
-        ind = tools.nearest_index(r, 1.0)  # Find the index where r == 1.0
-        x0_array = np.ones_like(r) * 0.1   # Setup the array of first guesses
-        x0_array[ind:] *= 20.0
-        velocity_r = so.newton(_eq_to_solve, x0=x0_array, args=(r,))
+        # array of first guesses `x0` whose values are 0.1 for r <= 1, and 2.1
+        # if r > 1.
+        v_init = np.array(r > 1, dtype=int) * 2 + 0.1
+        velocity_r = so.newton(_eq_to_solve, x0=v_init, args=(r,))
     # If r is float, just do a simple if statement
     else:
         if r <= 1.0:
@@ -327,15 +324,14 @@ def structure_tidal(r, sound_speed_0, radius_sonic_point, planet_mass,
         return eq
 
     if isinstance(r, np.ndarray):
-        # One line verison of Leo's initial value hack gives 0.1 below sonic
+        # One line version of Leo's initial value hack gives 0.1 below sonic
         # point and 2 above
         v_init = (np.array(r > 1, dtype=int) * 2 + 0.1)
 
         # Compute velocity profile
-        velocity_r = np.array([so.newton(_eq_to_solve, x0=v_init[k],
-            args=(r[k], sound_speed_0, radius_sonic_point, planet_mass,
-                  star_mass, semi_major_axis),
-            maxiter = 1000) for k in range(len(r))])    
+        velocity_r = so.newton(_eq_to_solve, x0=v_init,
+            args=(r, sound_speed_0, radius_sonic_point, planet_mass,
+                  star_mass, semi_major_axis), maxiter=1000)
     elif r <= 1.0:
         velocity_r = so.newton(_eq_to_solve, x0=1E-1,
                                args=(r, sound_speed_0, radius_sonic_point,
